module PostcodeInfo
  class Postcode
    # these attributes are set on construction
    attr_reader :as_given, :normalised
    attr_accessor :client
    # these attributes require lookups
    # attr_reader :latitude, :longitude, :addresses, :local_authority

    def initialize(given_postcode, client)
      @as_given = given_postcode
      @normalised = self.class.normalise(@as_given)
      @client = client
      @info = nil
    end

    def self.normalise(postcode)
      postcode.downcase.gsub(/[^a-z0-9]/, '')
    end

    def latitude
      lookup_info! if @info.nil?
      @latitude
    end

    def longitude
      lookup_info! if @info.nil?
      @longitude
    end

    def local_authority
      lookup_info! if @info.nil?
      @local_authority
    end
    
    def country
      lookup_info! if @info.nil?
      @country
    end

    def addresses
      @addresses ||= @client.addresses(@normalised)
    end
    alias_method :lookup_addresses!, :addresses

    def valid?
      @valid ||= @client.valid?(@normalised)
    end
    
    def lookup_info!
      @info = @client.info(@normalised)
      parse_info!
    end


    protected

      def set_coordinates!(info)
        if info[:centre]
          @latitude = info[:centre][:coordinates][0]
          @longitude = info[:centre][:coordinates][1]
        end
      end

      def parse_info!
        set_coordinates!(@info)
        @local_authority = @info[:local_authority]
        @country = @info[:country]
        @valid = true
      end
  end
end